let scrollDuration = 400;
let renderDuration = 700;

describe('Events', function() {
  it('scrolled event', function () {
    cy.visit('http://localhost:8080/cypress/fixtures/articles/page1.html?cypress=1');

    let scrollListenerSpy = {
      foo() {
      },
    };

    cy.spy(scrollListenerSpy, 'foo');

    cy.window().then(function(win) {
      win.ias = new win.IAS('.surface-container', {
        item: '.article',
        next: '.pager__next',
      });

      win.ias.on('scrolled', scrollListenerSpy.foo);
    });

    cy.scrollTo('bottom', {duration: scrollDuration}).then(function() {
      expect(scrollListenerSpy.foo).to.be.called;
    });
  });

  it('binded & unbinded events', function () {
    cy.visit('http://localhost:8080/cypress/fixtures/articles/page1.html?cypress=1');

    let scrollListenerSpy = {
      binded() {
      },
      unbinded() {
      },
    };

    cy.spy(scrollListenerSpy, 'binded');
    cy.spy(scrollListenerSpy, 'unbinded');

    cy.window().then(function(win) {
      win.ias = new win.IAS('.surface-container', {
        item: '.article',
        next: '.pager__next',
      });

      win.ias.on('binded', scrollListenerSpy.binded);
      win.ias.on('unbinded', scrollListenerSpy.unbinded);

      win.ias.unbind();
      win.ias.bind();

      expect(scrollListenerSpy.binded).to.be.called.once;
      expect(scrollListenerSpy.unbinded).to.be.called.once;
    });
  });

  it('load, loaded, append & appended events', function () {
    cy.visit('http://localhost:8080/cypress/fixtures/articles/page1.html?cypress=1');

    let scrollListenerSpy = {
      load() {
      },
      loaded() {
      },
      append() {
      },
      appended() {
      },
    };

    cy.spy(scrollListenerSpy, 'load');
    cy.spy(scrollListenerSpy, 'loaded');
    cy.spy(scrollListenerSpy, 'append');
    cy.spy(scrollListenerSpy, 'appended');

    cy.window().then(function(win) {
      win.ias = new win.IAS('.surface-container', {
        item: '.article',
        next: '.pager__next',
      });

      win.ias.on('load', scrollListenerSpy.load);
      win.ias.on('loaded', scrollListenerSpy.loaded);
      win.ias.on('append', scrollListenerSpy.append);
      win.ias.on('appended', scrollListenerSpy.appended);

      win.ias.next();

      // wait for it to render
      cy.wait(renderDuration).then(function() {
        expect(scrollListenerSpy.load).to.be.called.once;
        expect(scrollListenerSpy.loaded).to.be.called.once;
        expect(scrollListenerSpy.append).to.be.called.once;
        expect(scrollListenerSpy.appended).to.be.called.once;
      });
    });
  });
});
