let ias;

describe('Pagination', function() {
  beforeEach(function() {
    // runs before each test in the block
    cy.visit('http://localhost:8080/cypress/fixtures/articles/page1.html?cypress=1');
  });

  let dataProvider = [
    // extended options
    {
      test: 'extended',
      options: {
        element: '.non-existing-pager',
        hide: true,
      }
    },
    // short options
    {test: 'short', options: '.non-existing-pager'}
  ];

  dataProvider.forEach((data) =>  {
    it('throws error if pagination element not found (' + data.test + ')', function() {
      cy.window().then(function(win) {
        const spy = cy.spy(win, 'IAS');

        try {
          win.ias = ias = new win.IAS('.surface-container', {
            item: '.article',
            next: '.pager__next',
            pagination: data.options,
          });
        } catch (e) {}

        expect(spy).to.have.thrown('Error');
      });
    });
  });

  dataProvider = [
    // extended options
    {
      test: 'extended',
      options: {
        element: '.non-existing-pager',
        hide: false,
      }
    },
    // short options
    {test: 'short', options: false}
  ];

  dataProvider.forEach(function(data) {
    it('does not throw error if hide is false (' + data.test + ')', function () {
      cy.window().then(function (win) {
        const spy = cy.spy(win, 'IAS');

        win.ias = ias = new win.IAS('.surface-container', {
          item: '.article',
          next: '.pager__next',
          pagination: data.options,
        });

        expect(spy).to.not.have.thrown('Error');
      });
    });
  });

  it('hides pagination', function() {
    cy.get('.pager').should('be.visible');

    cy.window().then(function (win) {
      win.ias = ias = new win.IAS('.surface-container', {
        item: '.article',
        next: '.pager__next',
        pagination: '.pager',
      });

      cy.get('.pager').should('not.be.visible');
    });
  });

  it('does not hide pagination when hide set to false', function() {
    cy.window().then(function (win) {
      win.ias = ias = new win.IAS('.surface-container', {
        item: '.article',
        next: '.pager__next',
        pagination: false,
      });

      cy.get('.pager').should('be.visible');
    });
  })
});
