let scrollDuration = 400;
let renderDuration = 50;

describe('Scroll', function() {
  context('Articles', function() {
    beforeEach(function() {
      // runs before each test in the block
      cy.visit('http://localhost:8080/cypress/fixtures/articles/page1.html?cypress=1');

      cy.window().then(function(win) {
        win.ias = new win.IAS('.surface-container', {
          item: '.article',
          next: '.pager__next',
          delay: 0
        });
      });
    });

    it('loads next pages on scroll', function() {
      cy.get('#article2').should('not.be.visible');
      cy.get('#article3').should('not.be.visible');

      cy.scrollTo('bottom', {duration: scrollDuration}).then(function() {
        cy.get('#article2').should('be.visible');
        cy.get('#article3').should('not.be.visible');
      });

      cy.scrollTo('bottom', {duration: scrollDuration}).then(function() {
        cy.get('#article3').should('be.visible');
      });
    });

    it('stops loading if next selector not found', function() {
      // page 1 to 2
      cy.scrollTo('bottom', {duration: scrollDuration});

      // wait for it to render
      cy.wait(renderDuration);

      // page 2 to 3
      cy.scrollTo('bottom', {duration: scrollDuration});

      // wait for it to render
      cy.wait(renderDuration);

      // page 3 to 4 (4 does not exist)
      cy.scrollTo('bottom', {duration: scrollDuration});

      cy.wait(renderDuration);

      cy.get('.article3').should('have.length', 1);
    });
  });

  context('Stress', function() {
    beforeEach(function() {
      // runs before each test in the block
      cy.visit('http://localhost:8080/cypress/fixtures/stress/page1.html?cypress=1');

      cy.window().then(function(win) {
        win.ias = new win.IAS('.blocks', {
          item: '.blocks__block',
          next: '.pager__next',
        });
      });
    });

    it('loads next pages on scroll', function() {
      cy.get('.block144').should('not.be.visible');

      cy.scrollTo('bottom', {duration: scrollDuration}).then(function() {
        cy.get('.block144').should('be.visible');
      });
    });

    it('stops loading if next selector not found', function() {
      // page 1 to 2
      cy.scrollTo('bottom', {duration: scrollDuration});

      // wait for it to render
      cy.wait(renderDuration);

      // page 2 to 3 (3 does not exist)
      cy.scrollTo('bottom', {duration: scrollDuration});

      // wait for it to render
      cy.wait(renderDuration);

      cy.get('.block144').should('have.length', 1);
    });
  });

  context('Javascript', function() {
    beforeEach(function() {
      // runs before each test in the block
      cy.visit('http://localhost:8080/cypress/fixtures/javascript/page1.html?cypress=1');
    });

    it('loads next pages on scroll', function() {
      cy.get('.block1').should('be.visible');

      cy.get('.block144').should('not.be.visible');
      cy.get('.block216').should('not.be.visible');
      cy.get('.block288').should('not.be.visible');

      cy.scrollTo('bottom', {duration: scrollDuration}).then(function() {
        cy.get('.block144').should('be.visible');
      });

      cy.scrollTo('bottom', {duration: scrollDuration}).then(function() {
        cy.get('.block216').should('be.visible');
      });

      cy.scrollTo('bottom', {duration: scrollDuration}).then(function() {
        cy.get('.block288').should('be.visible');
      });
    });
  });
});
