import apprise from 'apprise';
import {getRootRect} from './dimensions';

function appriseFactory(options) {
  let notifier = apprise(options);

  notifier.on('exit', function(node) {
    node.parentNode.removeChild(node);
  });
  notifier.on('enter', function(node) {
    node.style.backgroundColor = '#eee';
    node.style.borderRadius = '4px';
    node.style.padding = '4px';
    node.style.margin = '10px';
    node.style.opacity = 1;
  });

  return notifier;
}

export class DebugPageProgress {
  constructor() {
    this.el = document.createElement('div');
    this.distance = 0;

    document.body.appendChild(this.el);
    this.el.style.position = 'fixed';
    this.el.style.backgroundImage = 'linear-gradient(to right, #FA612E, #ED2173)'; //'#F33B4E';
    this.el.opacity = 1;
    this.el.style.zIndex = 9990000;
    this.el.style.top = '0px';
    this.el.style.left = '0px';
    this.el.style.width = '0px';
    this.el.style.height = '3px';
    this.el.style.transition = 'width 0.1s';
  }

  render() {
    let overlayDebug = this.el;
    overlayDebug.style.width = this.distance + 'px';
  }

  update(min, max, current) {
    let root = getRootRect(window);
    let percent = (current - min) / (max - min);

    let self = this;
    this.distance = root.width * percent;

    self.render();
  }
}

export default class Debugger {
  constructor(ias) {
    this.ias = ias;
    this.notifier = appriseFactory({top: true, right: true});
    this.scrollNotifier = appriseFactory({bottom: true, right: true});
    this.scrollNode = null;

    this.progress = new DebugPageProgress();
  }

  bind() {
    let self = this;
    let printEvent = function(label) {
      return function() {
        let node = self.notifier(3000);

        node.innerHTML = label;
      };
    };

    this.ias.on('init', function() {
      this.lastElement().style.outline = 'solid 2px red';
    });

    this.ias.on('next', printEvent('next'));
    this.ias.on('load', printEvent('load'));
    this.ias.on('loaded', printEvent('loaded'));
    this.ias.on('append', printEvent('append'));
    this.ias.on('appended', printEvent('appended'));

    this.ias.on('append', function() {
      if (!this.lastElement()) {
        return;
      }

      this.lastElement().style.outline = 'none';
    });

    this.ias.on('appended', function() {
      this.lastElement().style.outline = 'solid 2px red';
    });

    this.ias.on('scroll', function(info) {
      this.render(info);
    }.bind(this));
  }

  render(info) {
    if (!this.scrollNode) this.scrollNode = this.scrollNotifier();

    this.scrollNode.innerHTML = '<pre style="margin:0;">' + JSON.stringify(info, undefined, '  ') + '</pre>';;

    this.progress.update(
        info.rootRect.height,
        info.bottom,
        info.scrollYBottom
    );
  }
}
