import $ from './el';
import extend from 'extend';

const defaults = {
  element: undefined,
  delay: 600,
  show: function(element) {
    element.style.opacity = 1;
  },
  hide: function(element) {
    element.style.opacity = 0;
  }
};

export default class Spinner {
  constructor(ias, options) {
    // no spinner wanted
    if (options === false) {
      return;
    }

    if (typeof options === 'string') {
      options = {
        element: options,
      }
    }

    let self = this;

    self.ias = ias;
    self.options = extend({}, defaults, options);
    self.element = $(self.options.element).get(); // @todo should we really cache this?
    self.hideFn = self.options.hide;
    self.showFn = self.options.show;

    if (!self.element) {
      throw new Error(`Element "${self.options.element}" not found`);
    }

    ias.on('binded', self.bind.bind(self));
  }

  bind() {
    let startTime, endTime, diff, delay, self = this;

    self.ias.on('next', function() {
      startTime = +new Date();

      self.show();
    });

    // setup delay
    self.ias.on('append', function(event) {
      endTime = +new Date();
      diff = endTime - startTime;

      delay = Math.max(0, self.options.delay - diff);

      // original executor
      let executor = event.executor;

      // wrap executor with delay
      event.executor = function(resolve) {
        setTimeout(function() {
          // turn hide function into promise
          Promise.resolve(self.hide()).then(() => {
            executor(resolve);
          });
        }, delay);
      };
    });

    self.hide();
  }

  show() {
    return Promise.resolve(this.showFn(this.element));
  }

  hide() {
    return Promise.resolve(this.hideFn(this.element));
  }
}
